# ------------------------------------------------------------ #
# ----------- LECTURA DE DATOS EN FORMA DE MATRIZ ------------ #
# -- Input:  fichero: nombre del fichero                -- #
# --         skip: n de lneas iniciales que se "saltan"   -- #
# --         dim1,dim2: n filas y de columnas de la matriz -- #
# --    Output: error: vale TRUE si no se abre el fichero   -- #
# --          datos: matriz leda (si no hubo error)        -- #
# ------------------------------------------------------------ #
leer_matriz=function(fichero,skip,dim1,dim2){
    abrir=open_file(fichero)
    if(!abrir$error){
        datos=matrix(scan(abrir$con,n=dim1*dim2,skip=skip,nlines=dim1,
                    quiet=T),dim1,dim2,byrow=TRUE)
        close(abrir$con)
        return(list(datos=datos,error=abrir$error))}
    return(list(error=abrir$error))}

# ------------------------------------------------------------ #
# -------- LECTURA DE DATOS EN FORMA DE "DOS TABLAS" --------- #
# -- Input:  fichero: nombre del fichero                -- #
# --         nrowsi: n de filas de la tabla i          -- #
# --         skipi: n lneas que se "saltan" de la tabla i -- #
# --         coli: columnas que se escogen de la tabla i    -- #
# --    Output: error: vale TRUE si no se abre el fichero   -- #
# --          datosi: tabla i leda (si no hubo error)  -- #
# ------------------------------------------------------------ #
leer_tablas=function(fichero,nrows1,skip1,col1,nrows2,skip2,col2){
    abrir=open_file(fichero)
    if(!abrir$error){
        datos1=as.matrix(read.table(abrir$con,nrows=nrows1,skip=skip1,fill=T,
            col.names=as.character(1:(length(col1)+6)))[,col1])
        datos2=as.matrix(read.table(abrir$con,nrows=nrows2,skip=skip2,fill=T,
            col.names=as.character(1:(length(col1)+6)))[,col2])
        close(abrir$con)
        return(list(datos1=datos1,datos2=datos2,error=abrir$error))}
    return(list(error=abrir$error))}

# ------------------------------------------------------------ #
# ------ LECTURA DEL FICHERO DE DATOS DE LA APLICACIN ------- #
# --    Output: error: vale TRUE si no se abre el fichero   -- #
# --          datos1,2,3: datos inmisin, emisin y mourela -- #
# ------------------------------------------------------------ #
lee_datosnuevos=function(fichero,dim_fecha,dim11,dim12,dim21,dim22,dim31,dim32,dim41,
                inmi_miss,emi_miss,emicc_miss,mour_miss){
    abrir=open_file(fichero)
    if(abrir$error) return(list("error"=TRUE))
# leer datos de fichero
    aux=scan(abrir$con,quiet=T)
    close(abrir$con)
    if(length(aux)!=(dim_fecha+2*(dim11*dim12+dim21*dim22+dim31*dim32+dim41))) return(list("error"=TRUE))
# fecha
    fecha=fechaPOSIX(aux[1:dim_fecha])
    aux=aux[-(1:dim_fecha)]
# inmisin
    datos1=matrix(aux[seq(1,2*dim11*dim12,2)],dim11,dim12)
    fiab1=matrix(aux[seq(2,2*dim11*dim12,2)],dim11,dim12)
    datos1[fiab1<=0]=rep(inmi_miss,dim12)[fiab1<=0]     #valores missing segn fiabilidad del dato
    aux=aux[-(1:(2*dim11*dim12))]
    # se generan valores missing para las estaciones eliminadas
    datos1=cbind(datos1[,1:7],matrix(inmi_miss,dim11,1),datos1[,8:10],matrix(inmi_miss,dim11,1),datos1[,11:12],matrix(rep(inmi_miss,2),dim11,2),datos1[,13])
    fiab1=cbind(fiab1[,1:7],matrix(rep(0,dim11),dim11,1),fiab1[,8:10],matrix(rep(0,dim11),dim11,1),fiab1[,11:12],matrix(rep(0,2*dim11),dim11,2),fiab1[,13])
# emisin central trmica: datos normalizados i.e. sobre seco (sin valores sobre bruto)
    datos2=matrix(aux[seq(1,2*dim21*dim22,2)],dim21,dim22)
    fiab2=matrix(aux[seq(2,2*dim21*dim22,2)],dim21,dim22)
    datos2[fiab2<=0]=rep(emi_miss,dim22)[fiab2<=0]   #valores missing segn fiabilidad del dato
    aux=aux[-(1:(2*dim21*dim22))]
# emisin ciclo combinado: datos sobre bruto
    datos3=matrix(aux[seq(1,2*dim31*dim32,2)],dim31,dim32)
    fiab3=matrix(aux[seq(2,2*dim31*dim32,2)],dim31,dim32)
	datos3[fiab3<=0]=rep(c(emicc_miss[1:8],-1,emicc_miss[9:10]),dim32)[fiab3<=0]	#valores missing segn fiabilidad del dato
    aux=aux[-(1:(2*dim31*dim32))]
    # se toma la potencia de la Turbina de vapor que se recibe con la Turbina de gas II
    if (datos3[9,2]<0){
        potTV_dato=-1}
    else{
        potTV_dato=datos3[9,2]} #Pot. Turbina de vapor
    potTV_fiab=fiab3[9,2]
    datos3=datos3[-9,]  #se quita la potencia de la Turbina de vapor
    fiab3=fiab3[-9,]
# meteorologa
    datos4=aux[seq(1,2*dim41,2)]
    fiab4=aux[seq(2,2*dim41,2)]
    datos4[fiab4<=0]=mour_miss[fiab4<=0]            #valores missing segn fiabilidad del dato

    return(list("error"=FALSE,"fecha"=fecha,"datos1"=datos1,"datos2"=datos2,"datos3"=datos3,"potTV"=potTV_dato,"datos4"=datos4))}
