# ---------------------------------------------------------------------- #
# ---------- CLCULO DE SUPERFICIES DE PREDICCIN DE INMISIN ---------- #
# ---------------------------------------------------------------------- #
SupInm=function(directorio){
# Librera de estadstica espacial
	library(geoR)
# Valores de configuracin
# nmero de estaciones
    n_est=17
# nombres de estaciones
	est=c("B1","B2","B6","B7","B8","C9","G5","E3","F2","F4","F5","F6","F8","G2","G4","H1","A6")
# localizaciones
	datx=c(108.7,117.3,124.7,132.0,111.0,121.1,77.5,96.5,97.3,87.9,82.4,93.1,102.5,92.6,91.6,102.1,99.5)
	daty=c(4824.9,4834.2,4829.2,4840.7,4828.3,4810.0,4830.2,4812.2,4820.0,4819.4,4812.9,4819.9,4823.8,4837.1,4829.1,4831.3,4850.3)
# rejilla: modificada para que cuadre con programa de visualizacin
	nrx=40
	nry=39
	minrx=67.95
	minry=4800.56
	maxrx=139.16
	maxry=4858.41
# modelo para el variograma, valores iniciales (por defecto) y sus "lmites"
	tipovar="matern"
	parini=list("cov.pars"=c(1.1470,0.3369),"nugget"=0.2376,"kappa"=111.7)
	esv.kc0=krige.control(cov.model=tipovar,obj.model=parini,lambda=1)
	#lmites para los parmetros estimados del matern
	parlim=matrix(c(0,0,0,0,6.25931,14.29007,1.81779,111.7750),byrow=T,nrow=2,ncol=4,
		dimnames=list(c("Inf","Sup"),c("Sigma","Phi","Nugget","Kappa")))
# fecha inicial
	fecha_ult=fechaPOSIX(c(2000,1,1,0,0))
# directorio de lectura de datos
	setwd(paste(directorio,"programas/x_medas",sep="/"))
# ------------------------------------------------------------
# Bucle de clculo continuo
# ------------------------------------------------------------
tiempo=0		# la primera vez no espera
while(TRUE){
# Esperar a que lleguen los datos
	cat("Esperando nuevos datos...\n")
	flush.console()
	Sys.sleep(tiempo)
	tiempo=30		# espera 3 minutos de cada vez
# Mirar si hay datos nuevos
	fichero=paste("predSM",est,".dat",sep="")
	abrir=open_file(fichero[1],"r+")
	if(!abrir$error){
		fecha_act=as.POSIXlt(strptime(paste(scan(abrir$con,what="",n=2,skip=1,nlines=1,quiet=T),collapse=" "),"%d/%b/%y %H:%M"))
		close(abrir$con)
		if(!is.na(fecha_act)&fecha_act>fecha_ult){
		# Nmero de datos "tericos"
			nobs=20
		# Leer ficheros PredSM
			t0=as.POSIXlt(Sys.time())
			cat("Lectura de archivos predSMxx.dat",fill=T)
			flush.console()
			datz=matrix(nrow=nobs,ncol=length(est),dimnames=list(1:nobs,est))
			predz=matrix(nrow=2,ncol=length(est),dimnames=list(nobs+0:1,est))
			for (i in 1:n_est){
				abrir=open_file(fichero[i],"r+")
				if(!abrir$error) {
					aux=scan(abrir$con,what="",n=69,skip=1,nlines=12,quiet=T)
					close(abrir$con)
					if(length(aux)==69){
						fecha_act=as.POSIXlt(strptime(paste(aux[1:2],collapse=" "),"%d/%b/%y %H:%M"))
						cat(format(fecha_act,"   %d/%b/%y %H:%M"),est[i],ifelse(i%%3,"\t","\n"))
						predz[2,i]=as.numeric(aux[length(aux)])
						aux=as.numeric(array(aux[-(1:3)],c(6,2,5))[,1,])
						datz[,i]=aux[length(aux)-((nobs-1):0)]}}}
			predz[1,]=datz[nobs,]
			datz[datz<0]=NA
			predz[predz<0]=NA
			cat("(",round(difftime(Sys.time(),t0,units="secs"),4),"s )\n")
			flush.console()
		# Necesito al menos dos valores no missing para krige.conv
			aux=apply(!is.na(predz),1,sum)
			if(any(aux>1)){
				esv.kc=esv.kc0
			# Si hay "bastantes" datos se estima,ajusta... (si no, valores por defecto esv.kc0)
				aux=apply(!is.na(datz),1,sum)
				datz=datz[aux>8,]
				nobs=ifelse(is.null(dim(datz)),0,dim(datz)[1])
				if(nobs>4){
				# Calcular los estimadores de los semivariogramas
					t0=Sys.time()
	     				cat("Estimacin del semivariograma\n")
					esv=variog(as.geodata(cbind(datx,daty,datz[1,])),option="cloud",
						estimator.type="modulus",messages=F)
				      for (k in 2:nobs){
						aux=variog(as.geodata(cbind(datx,daty,datz[k,])),option="cloud",
							estimator.type="modulus",messages=F)
						esv$u=c(esv$u,aux$u)
						esv$v=c(esv$v,aux$v)}
				# Ajustar modelo
					cat("y ajuste de un modelo")
					esv.var=c()
					# Modelo pure.nugget
					aux=try(variofit(esv,cov.model="pure.nugget",messages=F),silent=T)
					if(all(class(aux)!="try-error")) esv.var=aux
					# Modelo matern
					aux=try(variofit(esv,ini.cov.pars=parini$cov.pars,nugget=parini$nugget,cov.model=tipovar,
						fix.kappa=F,kappa=parini$kappa,messages=F),silent=T)
					if(all(class(aux)!="try-error")) {
						if (!is.list(esv.var)) {esv.var=aux}
						else {if(esv.var$value>=aux$value) esv.var=aux}}
					aux=try(variofit(esv,ini.cov.pars=parini$cov.pars,nugget=parini$nugget,cov.model=tipovar,fix.kappa=F,messages=F),silent=T)
					if(all(class(aux)!="try-error")) {
						if (!is.list(esv.var)) {esv.var=aux}
						else {if(esv.var$value>=aux$value) esv.var=aux}}
					# Si hay errores, se cogen valores iniciales
					if (is.list(esv.var)) {
						aux=c(esv.var$cov.pars,esv.var$nugget,esv.var$kappa)
						if(esv.var$cov.model=="pure.nugget"|all(parlim[1,]<=aux&aux<=parlim[2,])){
							esv.kc=krige.control(obj.model=esv.var)}}
					cat("(",round(difftime(Sys.time(),t0,units="secs"),4),"s )\n")
					flush.console()}
			# Calcular las superficies correspondientes a la ltima observacin y a las predicciones.
				t0=Sys.time()
				for (k in 1:2){
					if(sum(!is.na(predz[k,]))<2){
						cat("Demasiados valores missing para calcular la superficie ",k,"de",2,".",fill=T)}
					else{
						aux=try(krige.conv(as.geodata(cbind(datx,daty,predz[k,])),loc=expand.grid
							(seq(minrx,maxrx,l=nrx),seq(minry,maxry,l=nry)),krige=esv.kc,output=list(message=F)),silent=T)
						if(any(class(aux)=="try-error")){
							aux=try(krige.conv(as.geodata(cbind(datx,daty,predz[k,])),loc=expand.grid
								(seq(minrx,maxrx,l=nrx),seq(minry,maxry,l=nry)),krige=esv.kc0,output=list(message=F)),silent=T)
							if(all(class(aux)!="try-error")) esv.kc=esv.kc0}
						if(any(class(aux)=="try-error")){
							cat("No se ha podido calcular la superficie ",k,"de",2,".",fill=T)}
						else{
							cat("Clculo de la superficie ",k,"de",2,".",fill=T)
							cat("Parmetros del modelo:\n",paste(c("\tSigma = ","\tPhi = ","\tNugget = ","\tKappa = "),
									round(c(esv.kc$cov.pars,esv.kc$nugget,esv.kc$kappa),4),sep=""),fill=T)
							aux=aux$predict
							aux[is.na(aux)|aux<0]=0
							aux=matrix(formato(aux,8,2),nrx,nry)
							# almacenar resultados: formato olectra
							fichero=paste("PRE",k-1,".SUP",sep="")
							if (!file.exists(fichero)) file.create(fichero)
							abrir=open_file(fichero,"w")
							cat("#",format(fecha_act,"%d/%b/%y %H:%M"),fill=T,file=abrir$con)
							cat("IRGRID",formato(nrx,5),formato(nry,5),fill=T,file=abrir$con)
							cat(-1,fill=T,file=abrir$con)
							cat(formato(seq(minrx,maxrx,l=nrx),8,2),sep="","\n",file=abrir$con)
							cat(formato(seq(minry,maxry,l=nry),8,2),sep="","\n",file=abrir$con)
							write.table(aux,file=abrir$con,quote=F,sep="",row.names=F,col.names=F)
							close(abrir$con)}}}
				cat("(",round(difftime(Sys.time(),t0,units="secs"),4),"s )\n")
				flush.console()
			# Copia los ficheros en directorio compartido
				aux=list.files(pattern=".SUP")
				file.copy(from=aux,to=paste("../../prediccion_NL/",aux,sep=""),overwrite=T)}
		# Actualizar fecha
			fecha_ult=fecha_act}}}}
