# ------------------------------------------------------------ #
# ------------ PREDICCIN ESTADSTICA DE INMISIN ------------ #
# ---------- datos recogidos de la aplicacin MEDAS ---------- #
# -------- CONFIGURACIN DE VALORES: configuracion.txt ------- #
# ------------------------------------------------------------ #
#Series de fechas (incluyen predicciones)
# fecha (POSIXlt):			fechaP[n_long+n_pred]
# fecha (character):		fecha[n_long+n_pred,dim_fecha]
#Series de datos reales
# inmisin (numeric):		inmi_real[n_long,n_inm,n_est]
# inmisin (character):		finmision[n_long,(n_inm+8),n_est]
# emisin (numeric):		emi_real[n_long,n_emi,n_grupo]
# emisin (character):		femision[n_long,n_grupo*n_emi]
# emisin_cc (numeric):		emicc_real[n_long,n_emicc,n_grupocc]
# emisin_cc (character):	femisioncc[n_long,n_grupocc*n_emicc]
# mourela (numeric):		mourela[n_long,n_mourela]
# mourela (character):		fmourela[n_long,n_mourela]
#Series de medias horarias de datos reales de inmisin
# media horaria (numeric):	inmi_medH[n_long,n_inm,n_est]
#Series de predicciones inmisin
# prediccin media horaria(numeric):	inmi_predMH[n_long+n_pred,n_inm,n_est]
# prediccin binaria (numeric):			pred_bin[n_long+n_pred,n_inm,n_est]
# matriz histrica para binaria:		mathis_bin[n1_mat,n2_mat,n_est]
#Cdigos alarmas
# lmites para los mensajes:	lim_men[n_mensaje+1]
# cdigos de mensajes:			mensaje_texto[n_mensaje+1]
# serie de mensajes:			mensaje[n_long+n_pred,n_est]
#Cdigos para datos missing
# inmisin missing:		inmi_miss[n_inm]
# emisin missing:		emi_miss[n_emi]
# emisin_cc missing:	emicc_miss[n_emicc]
# mourela missing:		mour_miss[n_mourela]
# mensaje missing:		mensaje_miss
#Directorios para ficheros
# directorios de destino:			dir_dest[n_dest]
# directorio de configuraciones:	direc_conf
# directorio de origen de datos:	direc_datos
#Nombres de estaciones y variables
# estaciones:				est_nombre[n_est]
# grupos:					grupo_nombre[n_grupo]
# grupos_cc:				grupocc_nombre[n_grupocc]
# variables inmisin:		inm_nombre[n_inm]
# variables emisin:		emi_nombre[n_emi]
# variables emisin_cc:		emicc_nombre[n_emicc]
# variables mourela:		mour_nombre[n_mourela]
#Nombres de ficheros histricos creados en el programa
# his1_fich: nombre fichero histrico de inmisin
# his2_fich: nombre fichero histrico de prediccin
# his3_fich: nombre fichero histrico de emisin
# his4_fich: nombre fichero histrico de emisin ciclo combinado
# his5_fich: nombre fichero histrico de meteorologa
# his6_fich: nombre fichero histrico de clasificacin
# ------------------------------------------------------------ #
predESTADmin=function(){
	cat(rep("-",55),"\n\t\t\tSIPEI","\n\t	 Configuracin Datos Minutales\n",rep("-",55),"\n",sep="")
# Librera para modelos GAM para la prediccin binaria
	library(GAMBoost)
# Librera para modelos GAM para la prediccin de la tendencia
	library(mgcv)
	set.seed(0)
# Leer fichero de configuracin (fichero de texto en directorio de programa)
	t0=Sys.time()
	fichero="configuracion.txt"
	if (!file.exists(fichero)) stop("No encuentra configuracion.txt\nEjecucin de predESTADmin finalizada\n")
	abrir=open_file(fichero)	
	n_long=scan(abrir$con,n=1,skip=3,nlines=1,quiet=T)			#Longitud de las series reales
	n_pred=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)			#N instantes a predecir
	n_est=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)			#N de estaciones de la red de inmisin
	n_grupo=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)			#N de grupos de emisin
	n_grupocc=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)		#N de grupos de emisin ciclo combinado
	n_inm=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)			#N variables de estaciones de inmisin
	n_emi=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)			#N variables de emisin
	n_emicc=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)			#N variables de emisin ciclo combinado
	n_mourela=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)		#N variables de estacin meteorolgica de Mourela
	dim_fecha=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)		#N variables de la fecha: Ao,Mes,Da,Hora,Minuto
	direc_conf=scan(abrir$con,what="",sep="\n",n=1,skip=1,nlines=1,strip.white=T,quiet=T)		#Directorio de configuraciones
	direc_datos=scan(abrir$con,what="",sep="\n",n=1,skip=1,nlines=1,strip.white=T,quiet=T)		#Directorio de origen de datos
	n_dest=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)			#N de directorios de destino para las predicciones
	dir_dest=scan(abrir$con,what="",sep="\n",n=1,skip=1,nlines=n_dest,strip.white=T,quiet=T)	#Directorios de destino
	direc_MathisSO2=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)			#Directorio de matrices histricas de SO2
	direc_MathisNOx=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)			#Directorio de matrices histricas de NOx
	NOxFondo_RData=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)			#Directorio y fichero del NOx de Fondo
	datos_fich=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Nombre del fichero de datos de MEDAS
	fecha_fich=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Nombre fichero de fecha actual
	emi_fich=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Nombre fichero de emisin
	emicc_fich=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Nombre fichero de emisin ciclo combinado
	mour_fich=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Nombre fichero de mourela
	clasif_fich=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Nombre fichero de clasificacin
	est_fich=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Encabezado nombre ficheros de estaciones
	his1_enc=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Encabezado fichero histrico de inmisin
	his2_enc=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Encabezado fichero histrico de prediccin
	his3_enc=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Encabezado fichero histrico de emisin
	his4_enc=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Encabezado fichero histrico de emisin ciclo combinado
	his5_enc=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Encabezado fichero histrico de meteorologa
	his6_enc=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Encabezado fichero histrico de clasificacin
	mathis_enc=scan(abrir$con,what="",n=1,skip=1,nlines=1,quiet=T)	#Encabezado matrices histricas para prediccin binaria
	n1_matSO2=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)
	n1_matNOx=scan(abrir$con,n=1,skip=0,nlines=1,quiet=T)
	n2_mat=scan(abrir$con,n=1,skip=0,nlines=1,quiet=T)		#Dimensiones de matrices histricas
	lim_bin=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)		#Lmite para prediccin binaria
	est_nombre=scan(abrir$con,what="",n=n_est,skip=1,nlines=1,quiet=T)					#Nombres de estaciones
	est_distancia=scan(abrir$con,n=n_est,skip=1,nlines=1,quiet=T)						#Distancia de estaciones
	est_direccion=scan(abrir$con,n=n_est,skip=1,nlines=1,quiet=T)						#Direccin de estaciones
	grupo_nombre=scan(abrir$con,,what="",sep="\t",n=n_grupo,skip=1,nlines=1,quiet=T)	#Nombres de grupos
	grupocc_nombre=scan(abrir$con,,what="",sep="\t",n=n_grupo,skip=1,nlines=1,quiet=T)	#Nombres de grupos ciclo combinado
	inm_nombre=scan(abrir$con,what="",n=n_inm,skip=1,nlines=1,quiet=T)					#Nombres de variables de inmisin
	emi_nombre=scan(abrir$con,what="",n=n_emi,skip=1,nlines=1,quiet=T)					#Nombres de variables de emisin
	emicc_nombre=scan(abrir$con,what="",n=n_emicc,skip=1,nlines=1,quiet=T)				#Nombres de variables de emisin ciclo combinado
	mour_nombre=scan(abrir$con,what="",n=n_mourela,skip=1,nlines=1,quiet=T)				#Nombres de variables de Mourela
	inmi_miss=scan(abrir$con,n=n_inm,skip=1,nlines=1,quiet=T)		#Valores missing de inmisin
	emi_miss=scan(abrir$con,n=n_emi,skip=1,nlines=1,quiet=T)		#Valores missing de emisin
	emicc_miss=scan(abrir$con,n=n_emicc,skip=1,nlines=1,quiet=T)	#Valores missing de emisin ciclo combinado
	mour_miss=scan(abrir$con,n=n_mourela,skip=1,nlines=1,quiet=T)	#Valores missing de mourela
	mensaje_miss=scan(abrir$con,n=2,skip=1,nlines=1,quiet=T)		#Valor missing de mensaje
	n_mensaje=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)			#N de mensajes, sin contar el missing
	aux=scan(abrir$con,what=list(numeric(0),numeric(0),""),sep="\t",n=3*(n_mensaje+1),skip=1,nlines=n_mensaje+1,quiet=T)
	lim_men=cbind(aux[[1]],aux[[2]])	#Lmites de los mensajes
	mensaje_texto=aux[[3]]				#Texto de los mensajes
	factorGR=scan(abrir$con,n=n_grupo,skip=1,nlines=n_grupo,quiet=T)	#Factores de correccin de grupos
	clasif_min=scan(abrir$con,n=2,skip=1,nlines=1,quiet=T)			#Valores mnimos para clasificacin
	clasif_men_miss=scan(abrir$con,n=2,skip=1,nlines=1,quiet=T)		#Valores missing de mensaje de clasificacin
	clasif_men_num=scan(abrir$con,n=1,skip=1,nlines=1,quiet=T)		#N de mensajes de clasificacin, sin contar el missing
	aux=scan(abrir$con,what=list(numeric(0),numeric(0),""),sep="\t",n=3*(clasif_men_num+1),skip=1,nlines=clasif_men_num+1,quiet=T)
	clasif_men_lim=cbind(aux[[1]],aux[[2]])							#Lmites de los mensajes de clasificacin
	clasif_men_text=aux[[3]]										#Texto de los mensajes de clasificacin
	close(abrir$con)
# Inicializar las variables numeric
	fechaP=rep(fechaPOSIX(c(2000,1,1,0,0)),n_long+n_pred)
	inmi_real=array(rep(inmi_miss,each=n_long),c(n_long,n_inm,n_est),
		dimnames=list(NULL,formato(inm_nombre,9),est_nombre))
	inmi_medH=array(rep(inmi_miss,each=n_long),c(n_long,n_inm,n_est),
		dimnames=list(NULL,formato(paste(inm_nombre,"_MH",sep=""),9),est_nombre))
	inmi_predMH=array(rep(inmi_miss,each=n_long+n_pred),c(n_long+n_pred,n_inm,n_est),
		dimnames=list(NULL,formato(c("Pred_SO2","Pred_NOx",rep("Pred",n_inm-2)),11),est_nombre))
	pred_bin=array(rep(inmi_miss,each=n_long+n_pred),c(n_long+n_pred,n_inm,n_est),
		dimnames=list(NULL,formato(c("PredBin_SO2","PredBin_NOx",rep("PredBin",n_inm-2)),16),est_nombre))
	mensaje=array(rep(mensaje_miss,each=n_long+n_pred),c(n_long+n_pred,2,n_est),
		dimnames=list(NULL,formato(c("CodM_SO2","CodM_NOx"),10),est_nombre))
	emi_real=array(rep(emi_miss,each=n_long),c(n_long,n_emi,n_grupo),
		dimnames=list(NULL,formato(emi_nombre,9),formato(grupo_nombre,9)))
	emicc_real=array(rep(emicc_miss,each=n_long),c(n_long,n_emicc,n_grupocc),
		dimnames=list(NULL,formato(emicc_nombre,9),formato(grupocc_nombre,9)))
	potTV_real=rep(-1,each=n_long)
	mourela=matrix(mour_miss,n_long,n_mourela,
		dimnames=list(NULL,formato(mour_nombre,9)))
	clasif_prob=array(rep(inmi_miss[1:2],n_est),c(2*2,n_est),
		dimnames=list(c("ProbCT_SO2","ProbCT_NOx","ProbCC_SO2","ProbCC_NOx"),est_nombre))
	clasif_men_cod=array(rep(clasif_men_miss,each=1),c(2*2,n_est),
		dimnames=list(c("CodCT_SO2","CodCT_NOx","CodCC_SO2","CodCC_NOx"),est_nombre))
# Inicializar las variables character
	fecha=formato(fechaPOSIX(fechaP,F),5,0)
	dimnames(fecha)=list(NULL,formato(c("Ao","Mes","Da","Hora","Min"),5))
	finmision=array(matrix(c(rep(c(formato(c(inmi_miss,inmi_miss[1:6]),9,2),formato(rep(inmi_miss[1],2),11,2),
				formato(rep(inmi_miss[2],2),16,2),formato(mensaje_miss[1],10),formato("",21),formato(mensaje_miss[2],10),
				formato("",21)),n_long),rep(c(formato(rep("",15)),formato(rep(inmi_miss[1],2),11,2),
				formato(rep(inmi_miss[2],2),16,2),formato(mensaje_miss[1],10),formato("",21),formato(mensaje_miss[1],10),
				formato("",21)),n_pred)),n_long+n_pred,n_inm+14,byrow=T),c(n_long+n_pred,n_inm+14,n_est))
	finmision[n_long+1:n_pred,1:(n_inm+6),]=formato("",9)
	dimnames(finmision)=list(NULL,c(dimnames(inmi_real)[[2]],dimnames(inmi_medH)[[2]][1:6],
			dimnames(inmi_predMH)[[2]][1],dimnames(inmi_predMH)[[2]][2],dimnames(pred_bin)[[2]][1],dimnames(pred_bin)[[2]][2],dimnames(mensaje)[[2]][1],
			formato("Mensaje_SO2",21),dimnames(mensaje)[[2]][2],formato("Mensaje_NOx",21)),dimnames(inmi_real)[[3]])
	femision=matrix(formato(emi_miss,9,2),n_long,n_emi*n_grupo,byrow=T,
		dimnames=list(NULL,rep(dimnames(emi_real)[[2]],n_grupo)))
	femisioncc=matrix(formato(emicc_miss,9,2),n_long,n_emicc*n_grupocc,byrow=T,
		dimnames=list(NULL,rep(dimnames(emicc_real)[[2]],n_grupocc)))
	fpotTV=rep(-1,each=n_long)
	fmourela=matrix(formato(mour_miss,9,2),n_long,n_mourela,byrow=T,
		dimnames=list(NULL,dimnames(mourela)[[2]]))
	fclasif_prob=array(matrix(formato(rep(inmi_miss[1:2],n_est),12,2),4,n_est,byrow=F),c(4,n_est),
		dimnames=list(dimnames(clasif_prob)[[1]],est_nombre))
	fclasificacion=array(rep(c(formato(inmi_miss[1:2],12,2),formato(clasif_men_miss,12),formato(rep("",2),12)),2),c(6*2,n_est))
	dimnames(fclasificacion)=list(c(dimnames(clasif_prob)[[1]][1:2],dimnames(clasif_men_cod)[[1]][1:2],"MenCT_SO2","MenCT_NOx",
		dimnames(clasif_prob)[[1]][3:4],dimnames(clasif_men_cod)[[1]][3:4],"MenCC_SO2","MenCC_NOx"),est_nombre)
# Inicializar modelos para prediccin binaria y la prediccin de la tendencia
	mod_bin_so2=as.list(rep(NA,n_est))
	names(mod_bin_so2)=est_nombre
	mod_bin_nox=as.list(rep(NA,n_est))
	names(mod_bin_nox)=est_nombre
	mod_tend_so2=as.list(rep(NA,n_est))
	names(mod_tend_so2)=est_nombre
	mod_tend_nox=as.list(rep(NA,n_est))
	names(mod_tend_nox)=est_nombre
# Lectura de matrices histricas del SO2
	setwd(direc_MathisSO2)
	cat("Matrices histricas del SO2\n")
	for (i in 1:n_est){
		fichero=paste(mathis_enc,est_nombre[i],".dat",sep="")
		cat("Leyendo ",fichero,"...\n"); flush.console()
		aux=leer_matriz(fichero,1,n1_matSO2,n2_mat)
		if (aux$error) {
			cat("Error al abrir la matriz histrica\n")}
		else {
			aux2=aux$datos[seq(1,8000,16),] #Cogemos menos datos para la prediccin binaria
			aux$datos[aux$datos==inmi_miss[1]]=NA
			aux2[aux2==inmi_miss[1]]=NA
			aux=aux$datos[!is.na(aux$datos[,1]),1:3] #slo si X(t-5) no missing
			aux2=aux2[!is.na(aux2[,1]),1:3] #slo si X(t-5) no missing
		#Modelo GAM para la prediccin de la tendencia del SO2
			y=aux[,3]
			x0=aux[,2]
			x1=aux[,2]-aux[,1]
			mod_tend_so2[[i]]=gam(y~s(x0)+s(x1))
		#Modelo GAM para la prediccin binaria
			mod_bin_so2[[i]]=GAMBoost(x=cbind(aux2[,2],aux2[,2]-aux2[,1]),
				y=as.numeric(aux2[,3]>lim_bin),family="binomial",stepno=10,
				calc.hat=F,calc.se=F)}}
# Lectura de matrices histricas del NOx
	setwd(direc_MathisNOx)
	cat("Matrices histricas del NOx\n")
	for (i in 1:n_est){
		fichero=paste(mathis_enc,est_nombre[i],".dat",sep="")
		cat("Leyendo ",fichero,"...\n"); flush.console()
		aux=leer_matriz(fichero,1,n1_matNOx,n2_mat)
		if (aux$error) {
			cat("Error al abrir la matriz histrica\n")}
		else {
			aux2=aux$datos[seq(1,4000,8),] #Cogemos menos datos para la prediccin binaria
			aux$datos[aux$datos==inmi_miss[2]]=NA
			aux2[aux2==inmi_miss[2]]=NA
			aux=aux$datos[!is.na(aux$datos[,1]),1:3] #slo si X(t-5) no missing
			aux2=aux2[!is.na(aux2[,1]),1:3] #slo si X(t-5) no missing
		#Modelo GAM para la prediccin de la tendencia del NOx
			y=aux[,3]
			x0=aux[,2]
			x1=aux[,2]-aux[,1]
			mod_tend_nox[[i]]=gam(y~s(x0)+s(x1))
		#Modelo GAM para la prediccin binaria
			mod_bin_nox[[i]]=GAMBoost(x=cbind(aux2[,2],aux2[,2]-aux2[,1]),
				y=as.numeric(aux2[,3]>lim_bin),family="binomial",stepno=10,
				calc.hat=F,calc.se=F)}}
# Datos para clasificacin: NOx de fondo
	load(NOxFondo_RData)
	NOx_hist=NOx_2006
	# Cambiar a directorio de datos
	setwd(direc_datos)
	# Leer fecha actual
	aux=leer_matriz(fecha_fich,0,1,dim_fecha)
	if (aux$error){
		cat("Error al leer fecha_actual\n")
		aux$datos=c(2000,1,1,0,0)}
	fecha_actual=fechaPOSIX(aux$datos)
# Leer ficheros minutales con medias horarias y predicciones
	for(i in 1:n_est){  #leer los ltimos n_long=240 datos minutales de cada estacin
		fichero=paste(est_fich,est_nombre[i],".dat",sep="")
		aux=leer_tablas(fichero,n_long,1,1:(dim_fecha+n_inm+12),n_pred,0,1:(dim_fecha+6))
		if(!aux$error){
			fechaP=fechaPOSIX(rbind(aux$datos1[,1:dim_fecha],aux$datos2[,1:dim_fecha]))
			inmi_real[,,i]=aux$datos1[,(dim_fecha+1):(dim_fecha+n_inm)]
			inmi_medH[,1:6,i]=aux$datos1[,(dim_fecha+n_inm+1):(dim_fecha+n_inm+6)]
			inmi_predMH[,1,i]=c(aux$datos1[,dim_fecha+n_inm+7],aux$datos2[,dim_fecha+1])
			inmi_predMH[,2,i]=c(aux$datos1[,dim_fecha+n_inm+8],aux$datos2[,dim_fecha+2])
			pred_bin[,1,i]=c(aux$datos1[,dim_fecha+n_inm+9],aux$datos2[,dim_fecha+3])
			pred_bin[,2,i]=c(aux$datos1[,dim_fecha+n_inm+10],aux$datos2[,dim_fecha+4])
			mensaje[,1,i]=c(aux$datos1[,dim_fecha+n_inm+11],aux$datos2[,dim_fecha+5])
			mensaje[,2,i]=c(aux$datos1[,dim_fecha+n_inm+12],aux$datos2[,dim_fecha+6])
			fecha[,]=formato(fechaPOSIX(fechaP,F),5)
			finmision[1:n_long,1:(n_inm+6),i]=formato(cbind(inmi_real[,,i],inmi_medH[,1:6,i]),9,2)
			finmision[,n_inm+7:8,i]=formato(inmi_predMH[,1:2,i],11,2)
			finmision[,n_inm+9:10,i]=formato(pred_bin[,1:2,i],16,2)
			finmision[,n_inm+11:12,i]=cbind(formato(mensaje[,1,i],10),formato(mensaje_texto[mensaje[,1,i]+1],21))
			finmision[,n_inm+13:14,i]=cbind(formato(mensaje[,2,i],10),formato(mensaje_texto[mensaje[,2,i]+1],21))}}
# Leer datos de emisin
	aux=leer_matriz(emi_fich,2,n_long,dim_fecha+n_emi*n_grupo)
	if(!aux$error){
		fechaP[1:n_long]=fechaPOSIX(aux$datos[,1:dim_fecha])
		emi_real[,,]=aux$datos[,-(1:dim_fecha)]
		fecha[1:n_long,]=formato(fechaPOSIX(fechaP[1:n_long],F),5)
		femision[,]=formato(aux$datos[,-(1:dim_fecha)],9,2)}
# Leer datos de emisin ciclo combinado
	aux=leer_matriz(emicc_fich,2,n_long,dim_fecha+n_emicc*n_grupocc+1)
	if(!aux$error){
		fechaP[1:n_long]=fechaPOSIX(aux$datos[,1:dim_fecha])
		emicc_real[,,]=aux$datos[,1:(n_emicc*n_grupocc)+dim_fecha]
		potTV_real=aux$datos[,dim_fecha+n_emicc*n_grupocc+1]
		fecha[1:n_long,]=formato(fechaPOSIX(fechaP[1:n_long],F),5)
		femisioncc[,]=formato(aux$datos[,1:(n_emicc*n_grupocc)+dim_fecha],9,2)
		fpotTV=formato(aux$datos[,dim_fecha+n_emicc*n_grupocc+1],9,2)}
# Leer datos de meteorologa
	aux=leer_matriz(mour_fich,1,n_long,dim_fecha+n_mourela)
	if(!aux$error){
		fechaP[1:n_long]=fechaPOSIX(aux$datos[,1:dim_fecha])
		mourela[,]=aux$datos[,dim_fecha+1:n_mourela]
		fecha[1:n_long,]=formato(fechaPOSIX(fechaP[1:n_long],F),5)
		fmourela[,]=formato(mourela,9,2)}
cat("Lectura de datos iniciales (",round(difftime(Sys.time(),t0,units="secs"),4),"s )\n")
flush.console()
# ------------------------------------------------------------
# Bucle Infinito
# ------------------------------------------------------------
tiempo=0	#la primera vez no espera
while(TRUE){
	cat("Esperando datos nuevos...\n")
	flush.console()
	Sys.sleep(tiempo)
	tiempo=30 
# Leer datos de aplicacin
	setwd(direc_datos)
	aux=lee_datosnuevos(datos_fich,dim_fecha,n_inm,n_est-4,n_emi,n_grupo,n_emicc+1,n_grupocc,n_mourela,inmi_miss,emi_miss,emicc_miss,mour_miss)
	if(!aux$error){	 # regresa al inicio del bucle si no ha ledo el fichero completo.
		fecha_nueva=aux$fecha
		inmi_nuevo=aux$datos1
		emi_nuevo=aux$datos2
		emicc_nuevo=aux$datos3
		potTV_nuevo=aux$potTV
		mour_nuevo=aux$datos4
	# Clculos si hay datos nuevos
		if (fecha_actual<fecha_nueva){  #datos nuevos
			t0=as.POSIXlt(Sys.time())
			cat("\tCalculando para fecha",format(fecha_nueva,"%d/%m/%y %H:%M "))
			flush.console()
		# Calculo instantes a atrasar (si son ms de 4 horas, he perdido los datos)
			n_instante=min(as.numeric(difftime(fecha_nueva,fecha_actual,units="mins")),n_long-1)
		# Actualizo la fecha
			fecha_actual=fecha_nueva
		# Atrasar las series el nmero de instantes correspondientes
			fechaP[1:(n_long+n_pred-n_instante)]=fechaP[(1+n_instante):(n_long+n_pred)]
			inmi_real[1:(n_long-n_instante),,]=inmi_real[(1+n_instante):n_long,,]
			inmi_medH[1:(n_long-n_instante),,]=inmi_medH[(1+n_instante):n_long,,]
			inmi_predMH[1:(n_long+n_pred-n_instante),,]=inmi_predMH[(1+n_instante):(n_long+n_pred),,]
			pred_bin[1:(n_long+n_pred-n_instante),,]=pred_bin[(1+n_instante):(n_long+n_pred),,]
			mensaje[1:(n_long+n_pred-n_instante),,]=mensaje[(1+n_instante):(n_long+n_pred),,]
			emi_real[1:(n_long-n_instante),,]=emi_real[(1+n_instante):n_long,,]
			emicc_real[1:(n_long-n_instante),,]=emicc_real[(1+n_instante):n_long,,]
			potTV_real[1:(n_long-n_instante)]=potTV_real[(1+n_instante):n_long]
			mourela[1:(n_long-n_instante),]=mourela[(1+n_instante):n_long,]
			fecha[1:(n_long+n_pred-n_instante),]=fecha[(1+n_instante):(n_long+n_pred),]
			finmision[1:(n_long+n_pred-n_instante),,]=finmision[(1+n_instante):(n_long+n_pred),,]
			femision[1:(n_long-n_instante),]=femision[(1+n_instante):n_long,]
			femisioncc[1:(n_long-n_instante),]=femisioncc[(1+n_instante):n_long,]
			fpotTV[1:(n_long-n_instante)]=fpotTV[(1+n_instante):n_long]
			fmourela[1:(n_long-n_instante),]=fmourela[(1+n_instante):n_long,]
		# Datos y fechas perdidos (los hay cuando n_instante>1)
			if(n_instante>1){
				fechaP[(n_long-n_instante+1):(n_long-1)]=seq(fechaP[n_long-n_instante],fecha_nueva,by="min")[2:n_instante]
				inmi_real[(n_long-n_instante+1):(n_long-1),,]=rep(inmi_miss,each=n_instante-1)
				inmi_medH[(n_long-n_instante+1):(n_long-1),,]=rep(inmi_miss,each=n_instante-1)
				inmi_predMH[(n_long+n_pred-n_instante+1):(n_long+n_pred-1),,]=rep(inmi_miss,each=n_instante-1)
				pred_bin[(n_long+n_pred-n_instante+1):(n_long+n_pred-1),,]=0
				mensaje[(n_long+n_pred-n_instante+1):(n_long+n_pred-1),,]=mensaje_miss
				emi_real[(n_long-n_instante+1):(n_long-1),,]=rep(emi_miss,each=n_instante-1)
				emicc_real[(n_long-n_instante+1):(n_long-1),,]=rep(emicc_miss,each=n_instante-1)
				potTV_real[(n_long-n_instante+1):(n_long-1)]=rep(-1,each=n_instante-1)
				mourela[(n_long-n_instante+1):(n_long-1),]=rep(mour_miss,each=n_instante-1)
				fecha[(n_long-n_instante+1):(n_long-1),]=formato(fechaPOSIX(fechaP[(n_long-n_instante+1):(n_long-1)],F),5)
				finmision[(n_long-n_instante+1):(n_long-1),1:(n_inm+6),]=rep(formato(c(inmi_miss,inmi_miss[1:6]),9,2),each=n_instante-1)
				finmision[(n_long+n_pred-n_instante+1):(n_long+n_pred-1),n_inm+7:14,]=
					rep(c(formato(rep(inmi_miss[1],2),11,2),formato(rep(inmi_miss[2],2),16,2),formato(mensaje_miss[1],10),formato("",21),formato(mensaje_miss[2],10),formato("",21)),each=n_instante-1)
				femision[(n_long-n_instante+1):(n_long-1),]=rep(formato(emi_miss,9,2),each=n_instante-1)
				femisioncc[(n_long-n_instante+1):(n_long-1),]=rep(formato(emicc_miss,9,2),each=n_instante-1)
				fpotTV[(n_long-n_instante+1):(n_long-1)]=rep(formato(-1,9,2),each=n_instante-1)
				fmourela[(n_long-n_instante+1):(n_long-1),]=rep(formato(mour_miss,9,2),each=n_instante-1)}
		# ltimo instante, datos actual
			fechaP[n_long]=fecha_nueva
			inmi_real[n_long,,]=inmi_nuevo
			inmi_medH[n_long,,]=inmi_miss
			inmi_predMH[n_long,,]=inmi_miss
			emi_real[n_long,,]=emi_nuevo
			emicc_real[n_long,,]=emicc_nuevo
			potTV_real[n_long]=potTV_nuevo
			mourela[n_long,]=mour_nuevo
			fecha[n_long,]=formato(fechaPOSIX(fechaP[n_long],F),5)
			finmision[n_long,1:n_inm,]=formato(inmi_nuevo,9,2)
			femision[n_long,]=formato(emi_nuevo,9,2)
			femisioncc[n_long,]=formato(emicc_nuevo,9,2)
			fpotTV[n_long]=formato(potTV_nuevo,9,2)
			fmourela[n_long,]=formato(mour_nuevo,9,2)
		# Calculo fecha de predicciones
			fechaP[n_long+1:n_pred]=seq(fechaP[n_long],by="min",length.out=n_pred+1)[-1]
			fecha[n_long+1:n_pred,]=formato(fechaPOSIX(fechaP[n_long+1:n_pred],F),5)
		# Calculo serie de medias
			n_datos=60				  #60 datos minutales (12 datos cincominutales)
			for (j in (n_long-n_instante+1):n_long){ #instantes para los que se calcula la media
				aux=inmi_real[max(1,j-(n_datos-1)):j,,] #los 60 datos minutales a tener en cuenta (si los hay)
				aux[aux==rep(inmi_miss,each=dim(aux)[1])]=NA
				aux=apply(aux,c(2,3),"mean",na.rm=T)
				aux[is.na(aux)]=rep(inmi_miss,n_est)[is.na(aux)]
				inmi_medH[j,,]=aux
				finmision[j,n_inm+1:6,]=formato(aux[1:6,],9,2)}
		# Clculo prediccin puntual a media hora de serie horaria
			ind=which(!is.na(inmi_medH[n_long,1,])) #estaciones en las que hay dato nuevo
			if(length(ind)!=0){
				pred=matrix(data=NA,nrow=2,ncol=n_est)
				for (i in ind){ #para cada estacin con dato
					#variable 1: so2
					datos1=inmi_medH[,1,i]
					datos1[datos1==inmi_miss[1]]=NA
					datos1=datos1[!is.na(datos1)]
					if(length(datos1)>60){
						pred[1,i]=prediccion(i,datos1,mod_tend_so2)
						pred[1,i][pred[1,i]<3]=3}
					if (is.na(pred[1,i])) pred[1,i]=inmi_miss[1]
					#variable 2: nox
					datos2=inmi_medH[,2,i]
					datos2[datos2==inmi_miss[2]]=NA
					datos2=datos2[!is.na(datos2)]
					if(length(datos2)>60){
						pred[2,i]=prediccion(i,datos2,mod_tend_nox)
						pred[2,i][pred[2,i]<2]=2}
					if (is.na(pred[2,i])) pred[2,i]=inmi_miss[2]
					inmi_predMH[n_long+n_pred,1:2,i]=pred[,i]
					finmision[n_long+n_pred,n_inm+7:8,i]=formato(pred[,i],11,2)}}
		# Calculo prediccin binaria a media hora de serie horaria
			pred_bin[n_long+n_pred,1,]=pred_binaria(inmi_medH[n_long-1:0,1,],pred_bin[n_long+n_pred,1,],inmi_miss[1],mod_bin_so2)
			pred_bin[n_long+n_pred,2,]=pred_binaria(inmi_medH[n_long-1:0,2,],pred_bin[n_long+n_pred,2,],inmi_miss[2],mod_bin_nox)
			finmision[n_long+n_pred,n_inm+9:10,]=formato(pred_bin[n_long+n_pred,1:2,],16,2)
		# Calcular mensaje de alarma del sistema
			mensaje[n_long+n_pred,1,]=alarma_calc(inmi_predMH[n_long+n_pred,1,],lim_men[,1],mensaje[n_long+n_pred-2:1,1,])
			mensaje[n_long+n_pred,2,]=alarma_calc(inmi_predMH[n_long+n_pred,2,],lim_men[,2],mensaje[n_long+n_pred-2:1,2,])
			finmision[n_long+n_pred,n_inm+11,]=formato(mensaje[n_long+n_pred,1,],10)
			finmision[n_long+n_pred,n_inm+13,]=formato(mensaje[n_long+n_pred,2,],10)
			finmision[n_long+n_pred,n_inm+12,]=formato(mensaje_texto[1+mensaje[n_long+n_pred,1,]],21)
			finmision[n_long+n_pred,n_inm+14,]=formato(mensaje_texto[1+mensaje[n_long+n_pred,2,]],21)
		# Calcular mensaje mximo dado por el sistema
			mensaje_max=mensaje_texto[1+max(mensaje[n_long+n_pred,,])]
		# Calcular el origen de un episodio
			clasif_prob[,]=-1
			ind_clasif=sort(unique(c(which(inmi_medH[n_long,1,]>clasif_min[1]),which(inmi_medH[n_long,2,]>clasif_min[2]))))
			if(length(ind)!=0 & length(ind_clasif)!=0){
				viento_direccion=array(-1,dim=c(30,n_est))
				viento_velocidad=array(-1,dim=c(30,n_est))
				t1=rep(-1,n_est)
				for (i in ind_clasif){
					t1[i]=n_long-15-60*trunc(est_distancia[i]/(3.6*median(mourela[(n_long-30+1):n_long,1],na.rm=T)))
					if((t1[i]+30)>n_long){t1[i]=n_long-30+1}
					if(t1[i]<1){t1[i]=1}
					viento_direccion[,i]=mourela[t1[i]:(t1[i]+30-1),2]
					viento_velocidad[,i]=mourela[t1[i]:(t1[i]+30-1),1]
				}
				aux=origen(as.matrix(inmi_medH[n_long,1:2,ind_clasif]),emi_real[(n_long-119):n_long,1:2,],emicc_real[(n_long-119):n_long,1:2,],
					as.matrix(NOx_hist[,ind_clasif]),est_nombre[ind_clasif],est_distancia[ind_clasif],est_direccion[ind_clasif],as.matrix(viento_direccion[,ind_clasif]),
					as.matrix(viento_velocidad[,ind_clasif]),3.6,2,2,120,clasif_min[1],clasif_min[2])
				for (i in ind_clasif){
					clasif_prob[,i]=round(aux$prob[,which(ind_clasif==i)],2)
					}}
			fclasif_prob=formato(clasif_prob,12,2)
			clasif_men_cod[c(1,3),]=apply(outer(clasif_prob[c(1,3),],clasif_men_lim[,1],">="),c(1,2),sum)-1
			clasif_men_cod[c(2,4),]=apply(outer(clasif_prob[c(2,4),],clasif_men_lim[,2],">="),c(1,2),sum)-1
			fclasificacion[,]=rbind(fclasif_prob[1:2,],formato(clasif_men_cod[1:2,],12),
				formato(clasif_men_text[1+clasif_men_cod[1,]],12),formato(clasif_men_text[1+clasif_men_cod[2,]],12),
				fclasif_prob[3:4,],formato(clasif_men_cod[3:4,],12),
				formato(clasif_men_text[1+clasif_men_cod[3,]],12),formato(clasif_men_text[1+clasif_men_cod[4,]],12))
		# Escribir fecha actual
			if (!file.exists(fecha_fich)) file.create(fecha_fich)
			cat(formato(fechaPOSIX(fecha_actual,F),5),sep="",fill=TRUE,file=fecha_fich)
		# Escribir datos de inmisin
			# escribir "min_*.dat" de cada estacin
			for (i in 1:n_est){
				fichero=paste(est_fich,est_nombre[i],".dat",sep="")
				if (!file.exists(fichero)) file.create(fichero)
				write.table(cbind(fecha,finmision[,c(1:20,22,21,23),i]),file=fichero,quote=F,sep="",row.names=F)}
			# escribir "InHisM*" archivo histrico de inmisin
			his1_fich=paste(his1_enc,format(fechaPOSIX(fecha[n_long,]),"%d%m.%y"),sep="")
			aux=c(fecha[n_long,],finmision[n_long,1:(n_inm+6),])
			if(!file.exists(his1_fich)){
				file.create(his1_fich)
				aux=rbind(c(dimnames(fecha)[[2]],rep(dimnames(finmision)[[2]][1:(n_inm+6)],n_est)),aux)
				dimnames(aux)=list(NULL,c(rep(formato("",5),dim_fecha),
					as.vector(rbind(formato(matrix(est_nombre,n_inm+6,n_est,byrow=T),9)))))
				write.table(aux,file=his1_fich,quote=F,sep="",row.names=F)}
			else{
				cat(aux,"\n",sep="",file=his1_fich,append=T)}
		#Escribir los datos de prediccin
			# escribir "PredHisM*" archivo histrico de prediccin
			his2_fich=paste(his2_enc,format(fechaPOSIX(fecha[n_long,]),"%d%m.%y"),sep="")
			aux=c(fecha[n_long,],finmision[n_long+n_pred,n_inm+7:14,])
			if(!file.exists(his2_fich)){
				file.create(his2_fich)
				aux=rbind(c(dimnames(fecha)[[2]],rep(dimnames(finmision)[[2]][n_inm+7:14],n_est)),aux)
				dimnames(aux)=list(NULL,c(rep(formato("",5),dim_fecha),
					as.vector(rbind(formato(matrix(est_nombre,2,n_est,byrow=T),11),
						formato(matrix(est_nombre,2,n_est,byrow=T),16),formato(est_nombre,10),
						formato(est_nombre,21),formato(est_nombre,10),formato(est_nombre,21)))))
				write.table(aux,file=his2_fich,quote=F,sep="",row.names=F)}
			else{
				cat(aux,"\n",sep="",file=his2_fich,append=T)}
		#Escribir origen episodio
			# escribir "ClasifM.dat"
			if (!file.exists(clasif_fich)) file.create (clasif_fich)
			enc=rbind(formato(as.vector(clasif_men_text),12),formato(seq(0,length(clasif_men_text)-1),12))
			abrir=open_file(clasif_fich,"w")
			write.table(enc,file=abrir$con,quote=F,sep="",row.names=F,col.names=F)
			aux=cbind(formato(est_nombre,2),formato(t(clasif_men_cod),12))
			dimnames(aux)=list(NULL,c(formato("",2),formato(dimnames(clasif_men_cod)[[1]],12)))
			write.table(aux,file=abrir$con,quote=F,sep="",row.names=F)
			close(abrir$con)
			# escribir "ClasHisM*" archivo histrico de clasificacin
			his6_fich=paste(his6_enc,format(fechaPOSIX(fecha[n_long,]),"%d%m.%y"),sep="")
			aux=c(fecha[n_long,],fclasificacion)
			if(!file.exists(his6_fich)){
				file.create(his6_fich)
				aux=rbind(c(dimnames(fecha)[[2]],rep(formato(dimnames(fclasificacion)[[1]],12),n_est)),aux)
				dimnames(aux)=list(NULL,c(rep(formato("",5),dim_fecha),as.vector(formato(matrix(est_nombre,6*2,n_est,byrow=T),12))))
				write.table(aux,file=his6_fich,quote=F,sep="",row.names=F)}
			else{
				cat(aux,"\n",sep="",file=his6_fich,append=T)}
		# Escribir datos de emisin
			# escribir "emisionM.dat"
			if (!file.exists(emi_fich)) file.create(emi_fich)
			enc=c(formato(rep("",dim_fecha),5),rep(dimnames(emi_real)[[3]],each=n_emi))
			abrir=open_file(emi_fich,"w")
			cat(enc,"\n",sep="",file=abrir$con)
			write.table(cbind(fecha[1:n_long,],femision),file=abrir$con,quote=F,sep="",row.names=F)
			close(abrir$con)
			# escribir "EmHisM*" archivo histrico de emisin
			his3_fich=paste(his3_enc,format(fechaPOSIX(fecha[n_long,]),"%d%m.%y"),sep="")
			aux=c(fecha[n_long,],femision[n_long,])
			if(!file.exists(his3_fich)){
				file.create(his3_fich)
				aux=rbind(names(aux),aux)
				abrir=open_file(his3_fich,"w")  #se abre slo escritura
				cat(paste(dimnames(emi_real)[[3]],formato(factorGR,9,5),sep=""),"\n",sep="",file=abrir$con)
				write.table(aux,file=abrir$con,quote=F,sep="",row.names=F,col.names=enc)
				close(abrir$con)}
			else{
				cat(aux,"\n",sep="",file=his3_fich,append=T)}
		# Escribir datos de emisin ciclo combinado
			# escribir "emisionccM.dat"
			if (!file.exists(emicc_fich)) file.create(emicc_fich)
			enc=c(formato(rep("",dim_fecha),5),rep(dimnames(emicc_real)[[3]],each=n_emicc),formato("T. Vapor",9))
			abrir=open_file(emicc_fich,"w")
			cat(enc,"\n",sep="",file=abrir$con)
			aux=cbind(fecha[1:n_long,],femisioncc,fpotTV)
			dimnames(aux)[[2]][26]=formato("Pot",9,2)
			write.table(aux,file=abrir$con,quote=F,sep="",row.names=F)
			close(abrir$con)
			# escribir "EmccHisM*" archivo histrico de emisin ciclo combinado
			his4_fich=paste(his4_enc,format(fechaPOSIX(fecha[n_long,]),"%d%m.%y"),sep="")
			aux=c(fecha[n_long,],femisioncc[n_long,],fpotTV[n_long])
			if(!file.exists(his4_fich)){
				file.create(his4_fich)
				aux=rbind(c(names(aux)[-26],formato("Pot",9)),aux)
				abrir=open_file(his4_fich,"w")  #se abre slo escritura
				write.table(aux,file=abrir$con,quote=F,sep="",row.names=F,col.names=enc)
				close(abrir$con)}
			else{
				cat(aux,"\n",sep="",file=his4_fich,append=T)}
		# Escribir datos de meteorologa
			# escribir "meteorogiaM.dat"
			if (!file.exists(mour_fich)) file.create(mour_fich)
			write.table(cbind(fecha[1:n_long,],fmourela),file=mour_fich,quote=F,sep="",row.names=F)
			# escribir "MeHisM*" archivo histrico de meteorologa
			his5_fich=paste(his5_enc,format(fechaPOSIX(fecha[n_long,]),"%d%m.%y"),sep="")
			aux=c(fecha[n_long,],fmourela[n_long,])
			if(!file.exists(his5_fich)){
				file.create(his5_fich)
				aux=rbind(names(aux),aux)
				write.table(aux,file=his5_fich,quote=F,sep="",row.names=F,col.names=F)}
			else{
				cat(aux,"\n",sep="",file=his5_fich,append=T)}
		# la prediccin espacial necesita predSM*.dat
			aux=esc_pred(fechaP[n_long],inmi_real[,1,],inmi_medH[,1,],inmi_predMH[,1,],n_long,n_pred,n_est)
		# Copia los ficheros a los directorios necesarios
			ficheros=c(paste(est_fich,est_nombre,".dat",sep=""),fecha_fich,emi_fich,mour_fich,his1_fich,his2_fich,his3_fich,his4_fich,his5_fich,clasif_fich,his6_fich)
			for (i in 1:n_dest){
				file.copy(from=ficheros,to=paste(dir_dest,ficheros,sep="/"),overwrite=T)}
		cat("(",round(difftime(Sys.time(),t0,units="secs"),4),"s )\n")
		flush.console()}	#hay datos nuevos
	}}
# ------------------------------------------------------------
# Fin Bucle Infinito
# ------------------------------------------------------------
}
